package java.util.concurrent.atomic;

import java.io.InvalidObjectException;
import java.io.ObjectInputStream;
import java.util.function.LongBinaryOperator;
import java.io.Serializable;

public class LongAccumulator extends Striped64 implements Serializable
{
    private static final long serialVersionUID = 7249069246863182397L;
    private final LongBinaryOperator function;
    private final long identity;
    
    public LongAccumulator(LongBinaryOperator function, long n) {
        this.function = function;
        this.identity = n;
        this.base = n;
    }
    
    public void accumulate(long n) {
        Striped64.Cell[] cells;
        long base;
        long applyAsLong;
        if ((cells = this.cells) != null || ((applyAsLong = this.function.applyAsLong(base = this.base, n)) != base && !this.casBase(base, applyAsLong))) {
            boolean n2 = true;
            int n3;
            Striped64.Cell cell;
            long value;
            long applyAsLong2;
            if (cells == null || (n3 = cells.length - 1) < 0 || (cell = cells[getProbe() & n3]) == null || (n2 = ((applyAsLong2 = this.function.applyAsLong(value = cell.value, n)) == value || cell.cas(value, applyAsLong2))) == 0) {
                this.longAccumulate(n, this.function, (boolean)n2);
            }
        }
    }
    
    public long get() {
        Striped64.Cell[] cells = this.cells;
        long n = this.base;
        if (cells != null) {
            for (int i = 0; i < cells.length; ++i) {
                Striped64.Cell cell;
                if ((cell = cells[i]) != null) {
                    n = this.function.applyAsLong(n, cell.value);
                }
            }
        }
        return n;
    }
    
    public void reset() {
        Striped64.Cell[] cells = this.cells;
        this.base = this.identity;
        if (cells != null) {
            for (int i = 0; i < cells.length; ++i) {
                Striped64.Cell cell;
                if ((cell = cells[i]) != null) {
                    cell.value = this.identity;
                }
            }
        }
    }
    
    public long getThenReset() {
        Striped64.Cell[] cells = this.cells;
        long n = this.base;
        this.base = this.identity;
        if (cells != null) {
            for (int i = 0; i < cells.length; ++i) {
                Striped64.Cell cell;
                if ((cell = cells[i]) != null) {
                    long value = cell.value;
                    cell.value = this.identity;
                    n = this.function.applyAsLong(n, value);
                }
            }
        }
        return n;
    }
    
    @Override
    public String toString() {
        return Long.toString(this.get());
    }
    
    @Override
    public long longValue() {
        return this.get();
    }
    
    @Override
    public int intValue() {
        return (int)this.get();
    }
    
    @Override
    public float floatValue() {
        return (float)this.get();
    }
    
    @Override
    public double doubleValue() {
        return (double)this.get();
    }
    
    private Object writeReplace() {
        return new LongAccumulator.SerializationProxy(this);
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws InvalidObjectException {
        throw new InvalidObjectException("Proxy required");
    }
    
    private static class SerializationProxy implements Serializable
    {
        private static final long serialVersionUID = 7249069246863182397L;
        private final long value;
        private final LongBinaryOperator function;
        private final long identity;
        
        SerializationProxy(LongAccumulator longAccumulator) {
            this.function = longAccumulator.function;
            this.identity = longAccumulator.identity;
            this.value = longAccumulator.get();
        }
        
        private Object readResolve() {
            LongAccumulator longAccumulator = new LongAccumulator(this.function, this.identity);
            longAccumulator.base = this.value;
            return longAccumulator;
        }
    }
}
